@@ -1617,6 +1617,107 @@ public:
         //!
         SCI_DELWORDRIGHTEND = 2518,
 
+        //! This message copies the selection.  If the selection is empty then
+        //! copy the line with the caret.
+        SCI_COPYALLOWLINE = 2519,
+
+        //! This message returns a pointer to the document text.  Any
+        //! subsequent message will invalidate the pointer.
+        SCI_GETCHARACTERPOINTER = 2520,
+
+        //!
+        SCI_SETKEYSUNICODE = 2521,
+
+        //!
+        SCI_GETKEYSUNICODE = 2522,
+
+        //!
+        SCI_INDICSETALPHA = 2523,
+
+        //!
+        SCI_INDICGETALPHA = 2524,
+
+        //!
+        SCI_SETEXTRAASCENT = 2525,
+
+        //!
+        SCI_GETEXTRAASCENT = 2526,
+
+        //!
+        SCI_SETEXTRADESCENT = 2527,
+
+        //!
+        SCI_GETEXTRADESCENT = 2528,
+
+        //!
+        SCI_MARKERSYMBOLDEFINED = 2529,
+
+        //!
+        SCI_MARGINSETTEXT = 2530,
+
+        //!
+        SCI_MARGINGETTEXT = 2531,
+
+        //!
+        SCI_MARGINSETSTYLE = 2532,
+
+        //!
+        SCI_MARGINGETSTYLE = 2533,
+
+        //!
+        SCI_MARGINSETSTYLES = 2534,
+
+        //!
+        SCI_MARGINGETSTYLES = 2535,
+
+        //!
+        SCI_MARGINTEXTCLEARALL = 2536,
+
+        //!
+        SCI_MARGINSETSTYLEOFFSET = 2537,
+
+        //!
+        SCI_MARGINGETSTYLEOFFSET = 2538,
+
+        //!
+        SCI_ANNOTATIONSETTEXT = 2540,
+
+        //!
+        SCI_ANNOTATIONGETTEXT = 2541,
+
+        //!
+        SCI_ANNOTATIONSETSTYLE = 2542,
+
+        //!
+        SCI_ANNOTATIONGETSTYLE = 2543,
+
+        //!
+        SCI_ANNOTATIONSETSTYLES = 2544,
+
+        //!
+        SCI_ANNOTATIONGETSTYLES = 2545,
+
+        //!
+        SCI_ANNOTATIONGETLINES = 2546,
+
+        //!
+        SCI_ANNOTATIONCLEARALL = 2547,
+
+        //!
+        SCI_ANNOTATIONSETVISIBLE = 2548,
+
+        //!
+        SCI_ANNOTATIONGETVISIBLE = 2549,
+
+        //!
+        SCI_ANNOTATIONSETSTYLEOFFSET = 2550,
+
+        //!
+        SCI_ANNOTATIONGETSTYLEOFFSET = 2551,
+
+        //!
+        SCI_ADDUNDOACTION = 2560,
+
         //!
         SCI_STARTRECORD = 3001,
 
@@ -1802,9 +1903,15 @@ public:
         //! An XPM format pixmap.
         SC_MARK_PIXMAP = 25,
 
-        //! A full rectangle.
+        //! A full rectangle (ie. the margin background).
         SC_MARK_FULLRECT = 26,
 
+        //! A left rectangle (ie. part of the margin background).
+        SC_MARK_LEFTRECT = 27,
+
+        //! The value is available for plugins to use.
+        SC_MARK_AVAILABLE = 28,
+
         //! Characters can be used as symbols by adding this to the ASCII value
         //! of the character.
         SC_MARK_CHARACTER = 10000
@@ -1840,7 +1947,13 @@ public:
 
         //! The margin's background color will be set to the default foreground
         //! color.
-        SC_MARGIN_FORE = 3
+        SC_MARGIN_FORE = 3,
+
+        //! The margin will display text.
+        SC_MARGIN_TEXT = 4,
+
+        //! The margin will display right justified text.
+        SC_MARGIN_RTEXT = 5
     };
 
     enum
@@ -1952,16 +2065,11 @@ public:
         SC_FOLDLEVELBASE = 0x00400,
         SC_FOLDLEVELWHITEFLAG = 0x01000,
         SC_FOLDLEVELHEADERFLAG = 0x02000,
-        SC_FOLDLEVELBOXHEADERFLAG = 0x04000,
-        SC_FOLDLEVELBOXFOOTERFLAG = 0x08000,
-        SC_FOLDLEVELCONTRACTED = 0x10000,
-        SC_FOLDLEVELUNINDENT = 0x20000,
         SC_FOLDLEVELNUMBERMASK = 0x00fff
     };
 
     enum
     {
-        SC_FOLDFLAG_BOX = 0x0001,
         SC_FOLDFLAG_LINEBEFORE_EXPANDED = 0x0002,
         SC_FOLDFLAG_LINEBEFORE_CONTRACTED = 0x0004,
         SC_FOLDFLAG_LINEAFTER_EXPANDED = 0x0008,
@@ -1989,6 +2097,13 @@ public:
         SC_CACHE_DOCUMENT = 3
     };
 
+    enum
+    {
+        ANNOTATION_HIDDEN = 0,
+        ANNOTATION_STANDARD = 1,
+        ANNOTATION_BOXED = 2
+    };
+
     enum
     {
         EDGE_NONE = 0,
@@ -2002,6 +2117,11 @@ public:
         SC_CURSORWAIT = 4
     };
 
+    enum
+    {
+        UNDO_MAY_COALESCE = 1
+    };
+
     enum
     {
         VISIBLE_SLOP = 0x01,
@@ -2041,7 +2161,10 @@ public:
         SC_STARTACTION = 0x2000,
         SC_MOD_CHANGEINDICATOR = 0x4000,
         SC_MOD_CHANGELINESTATE = 0x8000,
-        SC_MODEVENTMASKALL = 0xffff
+        SC_MOD_CHANGEMARGIN = 0x10000,
+        SC_MOD_CHANGEANNOTATION = 0x20000,
+        SC_MOD_CONTAINER = 0x40000,
+        SC_MODEVENTMASKALL = 0x7ffff
     };
 
     enum
@@ -2347,7 +2470,40 @@ public:
         SCLEX_ASYMPTOTE = 85,
 
         //! Select the R lexer.
-        SCLEX_R = 86
+        SCLEX_R = 86,
+
+        //! Select the MagikSF lexer.
+        SCLEX_MAGIK = 87,
+
+        //! Select the PowerShell lexer.
+        SCLEX_POWERSHELL = 88,
+
+        //! Select the MySQL lexer.
+        SCLEX_MYSQL = 89,
+
+        //! Select the gettext .po file lexer.
+        SCLEX_PO = 90,
+
+        //! Select the TAL lexer.
+        SCLEX_TAL = 91,
+
+        //! Select the COBOL lexer.
+        SCLEX_COBOL = 92,
+
+        //! Select the TACL lexer.
+        SCLEX_TACL = 93,
+
+        //! Select the Sorcus lexer.
+        SCLEX_SORCUS = 94,
+
+        //! Select the PowerPro lexer.
+        SCLEX_POWERPRO = 95,
+
+        //! Select the Nimrod lexer.
+        SCLEX_NIMROD = 96,
+
+        //! Select the SML lexer.
+        SCLEX_SML = 97
     };
 
     //! Construct an empty QsciScintillaBase with parent \a parent, name \a
@@ -2429,12 +2585,23 @@ signals:
     //! deselected.
     void QSCN_SELCHANGED(bool yes);
 
+    //! This signal is emitted when the user cancels an auto-completion list.
+    //!
+    //! \sa SCN_AUTOCSELECTION()
+    void SCN_AUTOCCANCELLED();
+
+    //! This signal is emitted when the user deletes a character when an
+    //! auto-completion list is active.
+    void SCN_AUTOCCHARDELETED();
+
     //! This signal is emitted when the user selects an item in an
     //! auto-completion list.  It is emitted before the selection is inserted.
     //! The insertion can be cancelled by sending an SCI_AUTOCANCEL message
     //! from a connected slot.
     //! \a selection is the text of the selection.
     //! \a position is the start position of the word being completed.
+    //!
+    //! \sa SCN_AUTOCCANCELLED()
     void SCN_AUTOCSELECTION(const char *selection, int position);
 
     //! This signal is emitted when the document has changed for any reason.
@@ -2507,7 +2674,7 @@ signals:
     void SCN_MARGINCLICK(int position, int modifiers, int margin);
 
     //!
-    void SCN_MODIFIED(int, int, const char *, int, int, int, int, int);
+    void SCN_MODIFIED(int, int, const char *, int, int, int, int, int, int, int);
 
     //! This signal is emitted when the user attempts to modify read-only
     //! text.
@@ -2538,7 +2705,7 @@ signals:
     //! This signal is emitted when a range of text needs to be syntax styled.
     //! The range is from the value returned by the SCI_GETENDSTYLED message
     //! and \a position.  It is only emitted if the currently selected lexer is
-    //! SCNLEX_CONTAINER.
+    //! SCLEX_CONTAINER.
     //!
     //! \sa SCI_COLOURISE, SCI_GETENDSTYLED
     void SCN_STYLENEEDED(int position);
