@@ -46,7 +46,8 @@
 // The ctor.
 QsciLexerHTML::QsciLexerHTML(QObject *parent, const char *name)
     : QsciLexer(parent, name),
-      fold_compact(true), fold_preproc(true), case_sens_tags(false)
+      fold_compact(true), fold_preproc(true), case_sens_tags(false),
+      fold_script_comments(false), fold_script_heredocs(false)
 {
 }
 
@@ -1018,6 +1019,8 @@ void QsciLexerHTML::refreshProperties()
     setCompactProp();
     setPreprocProp();
     setCaseSensTagsProp();
+    setScriptCommentsProp();
+    setScriptHeredocsProp();
 }
 
 
@@ -1049,6 +1052,20 @@ bool QsciLexerHTML::readProperties(QSettings &qs,const QString &prefix)
     else
         rc = false;
 
+    flag = qs.readBoolEntry(prefix + "foldscriptcomments", false, &ok);
+
+    if (ok)
+        fold_script_comments = flag;
+    else
+        rc = false;
+
+    flag = qs.readBoolEntry(prefix + "foldscriptheredocs", false, &ok);
+
+    if (ok)
+        fold_script_heredocs = flag;
+    else
+        rc = false;
+
     return rc;
 }
 
@@ -1067,6 +1084,12 @@ bool QsciLexerHTML::writeProperties(QSettings &qs,const QString &prefix) const
     if (!qs.writeEntry(prefix + "casesensitivetags", case_sens_tags))
         rc = false;
 
+    if (!qs.writeEntry(prefix + "foldscriptcomments", fold_script_comments))
+        rc = false;
+
+    if (!qs.writeEntry(prefix + "foldscriptheredocs", fold_script_heredocs))
+        rc = false;
+
     return rc;
 }
 
@@ -1133,8 +1156,54 @@ void QsciLexerHTML::setFoldPreprocessor(bool fold)
 }
 
 
-// Set the "fold.preprocessor" property.
+// Set the "fold.html.preprocessor" property.
 void QsciLexerHTML::setPreprocProp()
 {
     emit propertyChanged("fold.html.preprocessor",(fold_preproc ? "1" : "0"));
 }
+
+
+// Return true if script comments can be folded.
+bool QsciLexerHTML::foldScriptComments() const
+{
+    return fold_script_comments;
+}
+
+
+// Set if script comments can be folded.
+void QsciLexerHTML::setFoldScriptComments(bool fold)
+{
+    fold_script_comments = fold;
+
+    setScriptCommentsProp();
+}
+
+
+// Set the "fold.hypertext.comment" property.
+void QsciLexerHTML::setScriptCommentsProp()
+{
+    emit propertyChanged("fold.hypertext.comment",(fold_script_comments ? "1" : "0"));
+}
+
+
+// Return true if script heredocs can be folded.
+bool QsciLexerHTML::foldScriptHeredocs() const
+{
+    return fold_script_heredocs;
+}
+
+
+// Set if script heredocs can be folded.
+void QsciLexerHTML::setFoldScriptHeredocs(bool fold)
+{
+    fold_script_heredocs = fold;
+
+    setScriptHeredocsProp();
+}
+
+
+// Set the "fold.hypertext.heredoc" property.
+void QsciLexerHTML::setScriptHeredocsProp()
+{
+    emit propertyChanged("fold.hypertext.heredoc",(fold_script_heredocs ? "1" : "0"));
+}
