@@ -45,6 +45,7 @@ extern "C++" {
 #include <qstringlist.h>
 
 #include <QByteArray>
+#include <QList>
 #include <QPointer>
 
 #include <Qsci/qsciglobal.h>
@@ -52,8 +53,14 @@ extern "C++" {
 #include <Qsci/qsciscintillabase.h>
 
 
-class QsciLexer;
+class QIODevice;
+class QPoint;
+
 class QsciCommandSet;
+class QsciLexer;
+class QsciStyle;
+class QsciStyledText;
+class ListBoxQt;
 
 
 //! \brief The QsciScintilla class implements a higher level, more Qt-like,
@@ -86,6 +93,18 @@ public:
         AiClosing = 0x04
     };
 
+    //! This enum defines the different annotation display styles.
+    enum AnnotationDisplay {
+        //! Annotations are not displayed.
+        AnnotationHidden = ANNOTATION_HIDDEN,
+
+        //! Annotations are drawn left justified with no adornment.
+        AnnotationStandard = ANNOTATION_STANDARD,
+
+        //! Annotations are surrounded by a box.
+        AnnotationBoxed = ANNOTATION_BOXED
+    };
+
     //! This enum defines the different sources for auto-completion lists.
     enum AutoCompletionSource {
         //! No sources are used, ie. automatic auto-completion is disabled.
@@ -189,6 +208,29 @@ public:
         BoxedTreeFoldStyle
     };
 
+    //! This enum defined the different margin types.
+    enum MarginType {
+        //! The margin contains symbols, including those used for folding.
+        SymbolMargin = SC_MARGIN_SYMBOL,
+
+        //! The margin contains symbols and uses the default foreground color
+        //! as its background color.
+        SymbolMarginDefaultForegroundColor = SC_MARGIN_FORE,
+
+        //! The margin contains symbols and uses the default background color
+        //! as its background color.
+        SymbolMarginDefaultBackgroundColor = SC_MARGIN_BACK,
+
+        //! The margin contains line numbers.
+        NumberMargin = SC_MARGIN_NUMBER,
+
+        //! The margin contains styled text.
+        TextMargin = SC_MARGIN_TEXT,
+
+        //! The margin contains right justified styled text.
+        TextMarginRightJustified = SC_MARGIN_RTEXT
+    };
+
     //! This enum defines the different pre-defined marker symbols.
     enum MarkerSymbol {
         //! A circle.
@@ -316,6 +358,45 @@ public:
     //! Destroys the QsciScintilla instance.
     virtual ~QsciScintilla();
 
+    //! Returns the API context, which is a list of words, before the position
+    //! \a pos in the document.  The context can be used by auto-completion and
+    //! call tips to help to identify which API call the user is referring to.
+    //! In the default implementation the current lexer determines what
+    //! characters make up a word, and what characters determine the boundaries
+    //! of words (ie. the start characters).  If there is no current lexer then
+    //! the context will consist of a single word.  On return \a context_start
+    //! will contain the position in the document of the start of the context
+    //! and \a last_word_start will contain the position in the document of the
+    //! start of the last word of the context.
+    virtual QStringList apiContext(int pos, int &context_start,
+            int &last_word_start);
+
+    //! Annotate the line \a line with the text \a text using the style number
+    //! \a style.
+    void annotate(int line, const QString &text, int style);
+
+    //! Annotate the line \a line with the text \a text using the style \a
+    //! style.
+    void annotate(int line, const QString &text, const QsciStyle &style);
+
+    //! Annotate the line \a line with the styled text \a text.
+    void annotate(int line, const QsciStyledText &text);
+
+    //! Annotate the line \a line with the list of styled text \a text.
+    void annotate(int line, const QList<QsciStyledText> &text);
+
+    //! Returns the annotation on line \a line, if any.
+    QString annotation(int line) const;
+
+    //! Returns the display style for annotations.
+    //!
+    //! \sa setAnnotationDisplay()
+    AnnotationDisplay annotationDisplay() const;
+
+    //! The annotations on line \a line are removed.  If \a line is negative
+    //! then all annotations are removed.
+    void clearAnnotations(int line = -1);
+
     //! Returns true if auto-completion lists are case sensitive.
     //!
     //! \sa setAutoCompletionCaseSensitivity()
@@ -576,6 +657,15 @@ public:
     //! at that position.
     int lineAt(const QPoint &pos) const;
 
+    //! QScintilla uses the combination of a line number and a character index
+    //! from the start of that line to specify the position of a character
+    //! within the text.  The underlying Scintilla instead uses a byte index
+    //! from the start of the text.  This will convert the \a position byte
+    //! index to the \a *line line number and \a *index character index.
+    //!
+    //! \sa positionFromLineIndex()
+    void lineIndexFromPosition(int position, int *line, int *index) const;
+
     //! Returns the length of line \a line int bytes or -1 if there is no such
     //! line.  In order to get the length in characters use text(line).length().
     int lineLength(int line) const;
@@ -595,7 +685,7 @@ public:
 
     //! Returns true if line numbers are enabled for margin \a margin.
     //!
-    //! \sa setMarginLineNumbers(), SCI_GETMARGINTYPEN
+    //! \sa setMarginLineNumbers(), marginType(), SCI_GETMARGINTYPEN
     bool marginLineNumbers(int margin) const;
 
     //! Returns the marker mask of margin \a margin.
@@ -608,6 +698,11 @@ public:
     //! \sa setMarginSensitivity(), marginClicked(), SCI_GETMARGINTYPEN
     bool marginSensitivity(int margin) const;
 
+    //! Returns the type of margin \a margin.
+    //!
+    //! \sa setMarginType(), SCI_GETMARGINTYPEN
+    MarginType marginType(int margin) const;
+
     //! Returns the width in pixels of margin \a margin.
     //!
     //! \sa setMarginWidth(), SCI_GETMARGINWIDTHN
@@ -698,6 +793,21 @@ public:
     //! \sa setPaper()
     QColor paper() const;
 
+    //! QScintilla uses the combination of a line number and a character index
+    //! from the start of that line to specify the position of a character
+    //! within the text.  The underlying Scintilla instead uses a byte index
+    //! from the start of the text.  This will return the byte index
+    //! corresponding to the \a line line number and \a index character index.
+    //!
+    //! \sa lineIndexFromPosition()
+    int positionFromLineIndex(int line, int index) const;
+
+    //! Reads the current document from the \a io device and returns true if
+    //! there was no error.
+    //!
+    //! \sa write()
+    bool read(QIODevice *io);
+
     //! Recolours the document between the \a start and \a end positions.
     //! \a start defaults to the start of the document and \a end defaults to
     //! the end of the document.
@@ -726,6 +836,12 @@ public:
     //! \sa resetFoldMarginColors()
     void setFoldMarginColors(const QColor &fore, const QColor &back);
 
+    //! Set the display style for annotations.  The default is
+    //! AnnotationStandard.
+    //!
+    //! \sa annotationDisplay()
+    void setAnnotationDisplay(AnnotationDisplay display);
+
     //! Enable the use of fill-up characters, either those explicitly set or
     //! those set by a lexer.  By default, fill-up characters are disabled.
     //!
@@ -799,6 +915,30 @@ public:
     //! \sa edgeMode()
     void setEdgeMode(EdgeMode mode);
 
+    //! Set the margin text of line \a line with the text \a text using the
+    //! style number \a style.
+    void setMarginText(int line, const QString &text, int style);
+
+    //! Set the margin text of line \a line with the text \a text using the
+    //! style \a style.
+    void setMarginText(int line, const QString &text, const QsciStyle &style);
+
+    //! Set the margin text of line \a line with the styled text \a text.
+    void setMarginText(int line, const QsciStyledText &text);
+
+    //! Set the margin text of line \a line with the list of styled text \a
+    //! text.
+    void setMarginText(int line, const QList<QsciStyledText> &text);
+
+    //! Set the type of margin \a margin to type \a type.
+    //!
+    //! \sa marginType(), SCI_SETMARGINTYPEN
+    void setMarginType(int margin, MarginType type);
+
+    //! The margin text on line \a line is removed.  If \a line is negative
+    //! then all margin text is removed.
+    void clearMarginText(int line = -1);
+
     //! Set the background colour, including the alpha component, of marker
     //! \a mnr to \a col.  If \a mnr is -1 then the colour of all markers is
     //! set.  The default is white.
@@ -884,7 +1024,7 @@ public:
     //! \sa setTabWidth()
     int tabWidth() const;
 
-    //! Returns the text edit's text.
+    //! Returns the text of the current document.
     //!
     //! \sa setText()
     QString text() const;
@@ -904,6 +1044,9 @@ public:
     //! \sa setWhitespaceVisibility()
     WhitespaceVisibility whitespaceVisibility() const;
 
+    //! Returns the word at the \a point screen coordinates.
+    QString wordAtPoint(const QPoint &point) const;
+
     //! Returns the set of valid word character as defined by the current
     //! language lexer.  If there is no current lexer then the set contains an
     //! an underscore, numbers and all upper and lower case alphabetic
@@ -917,6 +1060,12 @@ public:
     //! \sa setWrapMode()
     WrapMode wrapMode() const;
 
+    //! Writes the current document to the \a io device and returns true if
+    //! there was no error.
+    //!
+    //! \sa read()
+    bool write(QIODevice *io) const;
+
 public slots:
     //! Appends the text \a text to the end of the text edit.  Note that the
     //! undo/redo history is cleared by this function.
@@ -1126,11 +1275,12 @@ public slots:
     //! \sa eolVisibility()
     virtual void setEolVisibility(bool visible);
 
-    //! Sets the folding style for margin 2 to \a fold.  The default is
-    //! NoFoldStyle (ie. folding is disabled).
+    //! Sets the folding style for margin \a margin to \a fold.  The default
+    //! style is NoFoldStyle (ie. folding is disabled) and the default margin
+    //! is 2.
     //!
     //! \sa folding()
-    virtual void setFolding(FoldStyle fold);
+    virtual void setFolding(FoldStyle fold, int margin = 2);
 
     //! Sets the indentation of line \a line to \a indentation characters.
     //!
@@ -1189,7 +1339,7 @@ public slots:
     //! Enables or disables, according to \a lnrs, the display of line numbers
     //! in margin \a margin.
     //!
-    //! \sa marginLineNumbers(), SCI_SETMARGINTYPEN
+    //! \sa marginLineNumbers(), setMarginType(), SCI_SETMARGINTYPEN
     virtual void setMarginLineNumbers(int margin, bool lnrs);
 
     //! Sets the marker mask of margin \a margin to \a mask.  Only those
@@ -1237,7 +1387,7 @@ public slots:
 
     //! Sets the selection which starts at position \a indexFrom in line
     //! \a lineFrom and ends at position \a indexTo in line \a lineTo.  The
-    //! cursor is moved to the end of the selection.
+    //! cursor is moved to position \a indexTo in \a lineTo.
     //!
     //! \sa getSelection()
     virtual void setSelection(int lineFrom, int indexFrom, int lineTo,
@@ -1397,7 +1547,8 @@ private slots:
     void handleCharAdded(int charadded);
     void handleMarginClick(int pos, int margin, int modifiers);
     void handleModified(int pos, int mtype, const char *text, int len,
-            int added, int line, int foldNow, int foldPrev);
+            int added, int line, int foldNow, int foldPrev, int token,
+            int annotationLinesAdded);
     void handlePropertyChange(const char *prop, const char *val);
     void handleSavePointReached();
     void handleSavePointLeft();
@@ -1413,6 +1564,8 @@ private slots:
     void handleUpdateUI();
 
 private:
+    typedef QByteArray ScintillaString;
+
     enum IndentState {
         isNone,
         isKeywordStart,
@@ -1429,8 +1582,6 @@ private:
     int findStyledWord(const char *text, int style, const char *words);
 
     void checkMarker(int &mnr);
-    long posFromLineIndex(int line, int index) const;
-    void lineIndexFromPos(long pos, int *line, int *index) const;
     int currentIndent() const;
     int indentWidth() const;
     bool doFind();
@@ -1441,6 +1592,7 @@ private:
             int visLevels = 0, int level = -1);
     void setFoldMarker(int marknr, int mark = SC_MARK_EMPTY);
     QString convertTextS2Q(const char *s) const;
+    ScintillaString convertTextQ2S(const QString &q) const;
     void setLexerStyle(int style);
     void setStylesFont(const QFont &f, int style);
 
@@ -1453,14 +1605,16 @@ private:
             bool single);
 
     int adjustedCallTipPosition(int ctshift) const;
-    QStringList contextWords(int &pos, int *last_word = 0);
     bool getSeparator(int &pos) const;
     QString getWord(int &pos) const;
     char getCharacter(int &pos) const;
     bool isStartChar(char ch) const;
 
     bool ensureRW();
-    void insertAtPos(const QString &text, long pos);
+    void insertAtPos(const QString &text, int pos);
+
+    ScintillaString styleText(const QList<QsciStyledText> &styled_text,
+            char **styles, int style_offset = 0);
 
     struct FindState
     {
@@ -1483,6 +1637,7 @@ private:
     int ctPos;
     bool selText;
     FoldStyle fold;
+    int foldmargin;
     bool autoInd;
     BraceMatch braceMode;
     AutoCompletionSource acSource;
@@ -1491,16 +1646,26 @@ private:
     const char *wchars;
     CallTipsStyle call_tips_style;
     int maxCallTips;
+    QStringList ct_entries;
+    int ct_cursor;
+    QList<int> ct_shifts;
     bool showSingle;
     QPointer<QsciLexer> lex;
     QsciCommandSet *stdCmds;
     QsciDocument doc;
-    bool modified;
     QColor nl_text_colour;
     QColor nl_paper_colour;
     QByteArray explicit_fillups;
     bool fillups_enabled;
 
+    // The following allow ListBoxQt to distinguish between an auto-completion
+    // list and a user list, and to return the full selection of an
+    // auto-completion list.
+    friend class ListBoxQt;
+
+    QString acSelection;
+    bool isAutoCompletionList() const;
+
     QsciScintilla(const QsciScintilla &);
     QsciScintilla &operator=(const QsciScintilla &);
 };
