@@ -73,17 +73,28 @@ struct QsciAPIsPrepared
     // The raw API information.
     QStringList raw_apis;
 
-    QStringList apiWords(int api_idx, const QStringList &wseps) const;
+    QStringList apiWords(int api_idx, const QStringList &wseps,
+            bool strip_image) const;
     static QString apiBaseName(const QString &api);
 };
 
 
 
 // Return a particular API entry as a list of words.
-QStringList QsciAPIsPrepared::apiWords(int api_idx, const QStringList &wseps) const
+QStringList QsciAPIsPrepared::apiWords(int api_idx, const QStringList &wseps,
+        bool strip_image) const
 {
     QString base = apiBaseName(raw_apis[api_idx]);
 
+    // Remove any embedded image reference if necessary.
+    if (strip_image)
+    {
+        int tail = base.indexOf('?');
+
+        if (tail >= 0)
+            base.truncate(tail);
+    }
+
     if (wseps.isEmpty())
         return QStringList(base);
 
@@ -168,8 +179,8 @@ void QsciAPIsWorker::run()
     // Sort the full list.
     prepared->raw_apis.sort();
 
-    QStringList wseps = proxy->lex->autoCompletionWordSeparators();
-    bool cs = proxy->lex->caseSensitive();
+    QStringList wseps = proxy->lexer()->autoCompletionWordSeparators();
+    bool cs = proxy->lexer()->caseSensitive();
 
     // Split each entry into separate words but ignoring any arguments.
     for (int a = 0; a < prepared->raw_apis.count(); ++a)
@@ -178,7 +189,7 @@ void QsciAPIsWorker::run()
         if (abort)
             break;
 
-        QStringList words = prepared->apiWords(a, wseps);
+        QStringList words = prepared->apiWords(a, wseps, true);
 
         for (int w = 0; w < words.count(); ++w)
         {
@@ -205,11 +216,10 @@ void QsciAPIsWorker::run()
 
 // The ctor.
 QsciAPIs::QsciAPIs(QsciLexer *lexer)
-    : QObject(lexer),
-      lex(lexer), worker(0), ctcursor(0), origin_len(0)
+    : QsciAbstractAPIs(lexer),
+      worker(0), origin_len(0)
 {
     prep = new QsciAPIsPrepared;
-    lexer->setAPIs(this);
 }
 
 
@@ -327,7 +337,7 @@ QStringList QsciAPIs::positionOrigin(const QStringList &context, QString &path)
     {
         QString word = context[i];
 
-        if (!lex->caseSensitive())
+        if (!lexer()->caseSensitive())
             word = word.toUpper();
 
         if (i < old_context.count() && old_context[i] != word)
@@ -346,7 +356,7 @@ QStringList QsciAPIs::positionOrigin(const QStringList &context, QString &path)
     // auto-completion.
     if (origin_len > 0)
     {
-        const QString wsep = lex->autoCompletionWordSeparators().first();
+        const QString wsep = lexer()->autoCompletionWordSeparators().first();
 
         int start_new = old_context.count();
         int end_new = new_context.count() - 1;
@@ -414,15 +424,15 @@ bool QsciAPIs::originStartsWith(const QString &path, const QString &wsep)
 
 
 // Add auto-completion words to an existing list.
-void QsciAPIs::autoCompletionList(const QStringList &context,
-        QStringList &wlist)
+void QsciAPIs::updateAutoCompletionList(const QStringList &context,
+        QStringList &list)
 {
     QString path;
     QStringList new_context = positionOrigin(context, path);
 
     if (origin_len > 0)
     {
-        const QString wsep = lex->autoCompletionWordSeparators().first();
+        const QString wsep = lexer()->autoCompletionWordSeparators().first();
         QStringList::const_iterator it = origin;
 
         unambiguous_context = path;
@@ -444,8 +454,8 @@ void QsciAPIs::autoCompletionList(const QStringList &context,
                 // Append the space, we know the origin is unambiguous.
                 w.append(' ');
 
-                if (!wlist.contains(w))
-                    wlist << w;
+                if (!list.contains(w))
+                    list << w;
             }
 
             ++it;
@@ -477,7 +487,7 @@ void QsciAPIs::autoCompletionList(const QStringList &context,
                     noc.truncate(op);
             }
 
-            wlist << noc;
+            list << noc;
         }
     }
 }
@@ -490,7 +500,7 @@ const QsciAPIs::WordIndexList *QsciAPIs::wordIndexOf(const QString &word) const
 
     // Indirect through the case dictionary if the language isn't case
     // sensitive.
-    if (lex->caseSensitive())
+    if (lexer()->caseSensitive())
         csword = word;
     else
     {
@@ -524,7 +534,7 @@ void QsciAPIs::lastCompleteWord(const QString &word, QStringList &with_context,
 // Add auto-completion words based on the last partial word entered.
 void QsciAPIs::lastPartialWord(const QString &word, QStringList &with_context, bool &unambig)
 {
-    if (lex->caseSensitive())
+    if (lexer()->caseSensitive())
     {
         QMap<QString, WordIndexList>::const_iterator it = prep->wdict.lowerBound(word);
 
@@ -556,11 +566,11 @@ void QsciAPIs::lastPartialWord(const QString &word, QStringList &with_context, b
 
 
 // Handle the selection of an entry in the auto-completion list.
-void QsciAPIs::autoCompletionSelected(const QString &sel)
+void QsciAPIs::autoCompletionSelected(const QString &selection)
 {
     // If the selection is an API (ie. it has a space separating the selected
     // word and the optional origin) then remember the origin.
-    QStringList lst = sel.split(' ');
+    QStringList lst = selection.split(' ');
 
     if (lst.count() != 2)
     {
@@ -597,13 +607,13 @@ void QsciAPIs::autoCompletionSelected(const QString &sel)
 void QsciAPIs::addAPIEntries(const WordIndexList &wl, bool complete,
         QStringList &with_context, bool &unambig)
 {
-    QStringList wseps = lex->autoCompletionWordSeparators();
+    QStringList wseps = lexer()->autoCompletionWordSeparators();
 
     for (int w = 0; w < wl.count(); ++w)
     {
         const WordIndex &wi = wl[w];
 
-        QStringList api_words = prep->apiWords(wi.first, wseps);
+        QStringList api_words = prep->apiWords(wi.first, wseps, false);
 
         int idx = wi.second;
 
@@ -644,13 +654,13 @@ void QsciAPIs::addAPIEntries(const WordIndexList &wl, bool complete,
 
 
 // Return the call tip for a function.
-QString QsciAPIs::callTips(const QStringList &context,
-        QsciScintilla::CallTipsStyle style, int maxnr, int commas,
-        int &ctshift)
+QStringList QsciAPIs::callTips(const QStringList &context, int commas,
+        QsciScintilla::CallTipsStyle style,
+        QList<int> &shifts)
 {
     QString path;
     QStringList new_context = positionOrigin(context, path);
-    QStringList wseps = lex->autoCompletionWordSeparators();
+    QStringList wseps = lexer()->autoCompletionWordSeparators();
     QStringList cts;
 
     if (origin_len > 0)
@@ -658,7 +668,7 @@ QString QsciAPIs::callTips(const QStringList &context,
         QStringList::const_iterator it = origin;
         QString prev;
 
-        // Work out the lenght of the context.
+        // Work out the length of the context.
         const QString &wsep = wseps.first();
         QStringList strip = path.split(wsep);
         strip.removeLast();
@@ -682,14 +692,11 @@ QString QsciAPIs::callTips(const QStringList &context,
 
         while (it != prep->raw_apis.end() && (*it).startsWith(path))
         {
-            if (maxnr > 0 && maxnr == cts.count())
-                break;
-
             QString w = (*it).mid(ctstart);
 
             if (w != prev && enoughCommas(w, commas))
             {
-                ctshifts << shift;
+                shifts << shift;
                 cts << w;
                 prev = w;
             }
@@ -708,7 +715,7 @@ QString QsciAPIs::callTips(const QStringList &context,
             for (int i = 0; i < wil->count(); ++i)
             {
                 const WordIndex &wi = (*wil)[i];
-                QStringList awords = prep->apiWords(wi.first, wseps);
+                QStringList awords = prep->apiWords(wi.first, wseps, true);
 
                 // Check the word is the function name and not part of any
                 // context.
@@ -725,80 +732,20 @@ QString QsciAPIs::callTips(const QStringList &context,
                 if (!enoughCommas(api, commas))
                     continue;
 
-                if (maxnr > 0 && maxnr == cts.count())
-                    break;
-
                 if (style == QsciScintilla::CallTipsNoContext)
                 {
-                    ctshifts << 0;
+                    shifts << 0;
                     cts << (fname + api.mid(tail));
                 }
                 else
                 {
-                    ctshifts << tail - fname.length();
+                    shifts << tail - fname.length();
                     cts << api;
                 }
             }
     }
 
-    // See if we want to add a down arrow.
-    if (maxnr < 0 && cts.count() > 1)
-    {
-        // Remember the state so we can scroll through it later.
-        ctlist = cts;
-        ctcursor = 0;
-
-        QString ct = cts[0];
-
-        ct.prepend('\002');
-
-        ctshift = ctshifts.first();
-
-        return ct;
-    }
-
-    // Find the biggest shift.
-    ctshift = 0;
-
-    for (int i = 0; i < ctshifts.count(); ++i)
-    {
-        int shift = ctshifts[i];
-
-        if (ctshift < shift)
-            ctshift = shift;
-    }
-
-    ctshifts.clear();
-    ctlist.clear();
-
-    return cts.join("\n");
-}
-
-
-// Return the next or previous call tip.
-QString QsciAPIs::callTipsNextPrev(int dir, int &ctshift)
-{
-    QString ct;
-
-    // Get the call tip.
-    if (dir == 1 && ctcursor > 0)
-        ct = ctlist[--ctcursor];
-    else if (dir == 2 && ctcursor < ctlist.count() - 1)
-        ct = ctlist[++ctcursor];
-
-    // Add the arrows.
-    if (!ct.isEmpty())
-    {
-        if (ctcursor < ctlist.count() - 1)
-            ct.prepend('\002');
-
-        if (ctcursor > 0)
-            ct.prepend('\001');
-
-        ctshift = ctshifts[ctcursor];
-    }
-
-    return ct;
+    return cts;
 }
 
 
@@ -888,7 +835,7 @@ bool QsciAPIs::loadPrepared(const QString &fname)
     char *lex_name;
     pds >> lex_name;
 
-    if (qstrcmp(lex_name, lex->lexer()) != 0)
+    if (qstrcmp(lex_name, lexer()->lexer()) != 0)
     {
         delete[] lex_name;
         return false;
@@ -899,7 +846,7 @@ bool QsciAPIs::loadPrepared(const QString &fname)
     prep->wdict.clear();
     pds >> prep->wdict;
 
-    if (!lex->caseSensitive())
+    if (!lexer()->caseSensitive())
     {
         // Build up the case dictionary.
         prep->cdict.clear();
@@ -940,7 +887,7 @@ bool QsciAPIs::savePrepared(const QString &fname) const
     // Use a serialisation format supported by Qt v3.0 and later.
     pds.setVersion(QDataStream::Qt_3_0);
     pds << PreparedDataFormatVersion;
-    pds << lex->lexer();
+    pds << lexer()->lexer();
     pds << prep->wdict;
     pds << prep->raw_apis;
 
@@ -992,7 +939,7 @@ QString QsciAPIs::prepName(const QString &fname, bool mkpath) const
         pdname = pd.filePath(qsci_dir);
     }
 
-    return QString("%1/%2.pap").arg(pdname).arg(lex->lexer());
+    return QString("%1/%2.pap").arg(pdname).arg(lexer()->lexer());
 }
 
 
@@ -1001,7 +948,7 @@ QStringList QsciAPIs::installedAPIFiles() const
 {
     QString qtdir = QLibraryInfo::location(QLibraryInfo::DataPath);
 
-    QDir apidir = QDir(QString("%1/qsci/api/%2").arg(qtdir).arg(lex->lexer()));
+    QDir apidir = QDir(QString("%1/qsci/api/%2").arg(qtdir).arg(lexer()->lexer()));
     QStringList fnames;
 
     QStringList filters;
